@@ -48,12 +48,16 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
         self.active_queries: Dict[str, Query] = dict()
 
         self.common_query_skills = []
+        self._deprecated_skills = []
+
         config = config or Configuration().get('intents', {}).get("common_query") or dict()
-        self._extension_time = config.get('extension_time') or 3
+        self._extension_time = config.get('extension_time') or 1
         CommonQAService._EXTENSION_TIME = self._extension_time
-        self._min_wait = config.get('min_response_wait') or 2
-        self._max_time = config.get('max_response_wait') or 6  # regardless of extensions
-        reranker_module = config.get("reranker", "ovos-choice-solver-bm25")  # default to BM25 from ovos-classifiers
+        self._min_wait = config.get('min_response_wait') or 1
+        self._max_time = config.get('max_response_wait') or 4  # regardless of extensions
+        self._min_self_confidence = config.get('min_self_confidence', 0.5)
+        self._min_reranker_score = config.get('min_reranker_score', 0.2)
+        reranker_module = config.get("reranker", "ovos-flashrank-reranker-plugin")
         self.reranker = None
         try:
             for name, plug in find_multiple_choice_solver_plugins().items():
@@ -65,7 +69,7 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
                 LOG.info("No CommonQuery ReRanker loaded!")
         except Exception as e:
             LOG.error(f"Failed to load ReRanker plugin: {e}")
-        self.ignore_scores = config.get("ignore_skill_scores", False) and self.reranker is not None
+        self.ignore_scores = config.get("ignore_skill_scores", True) and self.reranker is not None
         self.add_event('question:query.response', self.handle_query_response)
         self.add_event('common_query.question', self.handle_question)
         self.add_event('ovos.common_query.pong', self.handle_skill_pong)
@@ -76,6 +80,11 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
         if message.data["skill_id"] not in self.common_query_skills:
             self.common_query_skills.append(message.data["skill_id"])
             LOG.debug("Detected CommonQuery skill: " + message.data["skill_id"])
+            if message.data.get("is_classic_cq", True):
+                deprecated_id = message.data["skill_id"]
+                LOG.warning(f"{deprecated_id} is using the deprecated CommonQuery skill class, "
+                            f"it might stop working in the near future")
+                self._deprecated_skills.append(deprecated_id)
 
     def is_question_like(self, utterance: str, lang: str) -> bool:
         """
@@ -90,8 +99,11 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
             LOG.debug("utterance has less than 3 words, doesnt look like a question")
             return False
         # skip utterances meant for common play
+        if self.voc_match(utterance, "Weather", lang):
+            LOG.debug("utterance has 'weather' keywords, doesnt look like a general knowledge question")
+            return False
         if self.voc_match(utterance, "Play", lang):
-            LOG.debug("utterance has 'playback' keywords, doesnt look like a question")
+            LOG.debug("utterance has 'playback' keywords, doesnt look like a general knowledge question")
             return False
         # require a "question word"
         return self.voc_match(utterance, "QuestionWord", lang)
@@ -121,13 +133,15 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
             LOG.info(f"Gathering answers from skills: {self.common_query_skills}")
 
         for utterance in utterances:
+            LOG.debug(f"is question: {self.is_question_like(utterance, lang)}")
             if self.is_question_like(utterance, lang):
                 message.data["lang"] = lang  # only used for speak method
                 message.data["utterance"] = utterance
                 answered, query = self.handle_question(message)
                 if answered and query.response_confidence >= self.config.get("min_conf", 0.01):
                     query.callback_data["conf"] = query.response_confidence
-                    match = IntentHandlerMatch(match_type='question:action',
+                    old_style = query.selected_skill in self._deprecated_skills
+                    match = IntentHandlerMatch(match_type='question:action' if old_style else f'question:action.{query.selected_skill}',
                                                match_data=query.callback_data,
                                                skill_id=query.selected_skill,
                                                utterance=utterance)
@@ -142,7 +156,8 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
         sess = SessionManager.get(message)
         query = Query(session_id=sess.session_id, query=utt, lang=sess.lang,
                       replies=[], extensions=[],
-                      query_time=time.time(), timeout_time=time.time() + self._max_time,
+                      query_time=time.time(),
+                      timeout_time=time.time() + self._max_time,
                       responses_gathered=Event(), completed=Event(),
                       answered=False,
                       queried_skills=[s for s in sess.blacklisted_skills
@@ -152,7 +167,7 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
         self.active_queries[sess.session_id] = query
         self.enclosure.mouth_think()
 
-        LOG.info(f'Searching for {utt}')
+        LOG.info(f"Searching for '{utt}' with max search time: {self._max_time}s")
         # Send the query to anyone listening for them
         msg = message.reply('question:query', data={'phrase': utt})
         if "skill_id" not in msg.context:
@@ -189,7 +204,8 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
             LOG.debug(f"ignoring match, skill_id '{skill_id}' blacklisted by Session '{sess.session_id}'")
             return
 
-        query = self.active_queries.get(SessionManager.get(message).session_id)
+        sess_id = SessionManager.get(message).session_id
+        query = self.active_queries.get(sess_id)
         if not query:
             LOG.warning(f"Late answer received from {skill_id}, no active query for: {search_phrase}")
             return
@@ -207,8 +223,8 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
             if answer:
                 LOG.info(f'Answer from {skill_id}')
                 query.replies.append(message.data)
-
-            query.queried_skills.append(skill_id)
+                if skill_id not in query.queried_skills:
+                    query.queried_skills.append(skill_id)
 
             # Remove the skill from list of timeout extensions
             if skill_id in query.extensions:
@@ -219,16 +235,6 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
             if self.common_query_skills and set(query.queried_skills) == set(self.common_query_skills):
                 LOG.debug("All skills answered")
                 query.responses_gathered.set()
-            else:
-                time_to_wait = (query.timeout_time - time.time())
-                if time_to_wait > 0:
-                    LOG.debug(f"Waiting up to {time_to_wait}s for other skills")
-                    query.responses_gathered.wait(time_to_wait)
-
-                # not waiting for any more skills
-                if not query.extensions and not query.responses_gathered.is_set():
-                    LOG.debug(f"Exiting early, no more skills to wait for session ({query.session_id})")
-                    query.responses_gathered.set()
 
     def _query_timeout(self, message: Message):
         """
@@ -251,6 +257,9 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
         best = None
         ties = []
         for response in query.replies:
+            if response['conf'] < self._min_self_confidence:
+                LOG.debug(f"Discarding {response['skill_id']} low confidence answer: {response['conf']} - {response['answer']}")
+                continue
             if response["skill_id"] in sess.blacklisted_skills:
                 continue
             if not self.ignore_scores:
@@ -265,28 +274,46 @@ class CommonQAService(PipelineStageMatcher, OVOSAbstractApplication):
                 ties.append(response)
 
         if best:
-            if len(ties) > 1:
-                tied_ids = [m["skill_id"] for m in ties]
+            tied_ids = set(m["skill_id"] for m in ties)
+            if len(tied_ids) > 1:
                 LOG.debug(f"Tied skills: {tied_ids}")
-                answers = {m["answer"]: m for m in ties}
-                if self.reranker is None:
+            answers = {m["answer"]: m for m in ties}
+            if self.reranker is None:
+                if len(tied_ids) > 1:
                     LOG.debug("No ReRanker available, selecting randomly")
-                    # random pick, no re-ranker available
-                    best_ans = list(answers.keys())[0]
-                else:
-                    reranked = self.reranker.rerank(query.query,
-                                                    list(answers.keys()),
-                                                    lang=query.lang)
-                    for score, ans in reranked:
-                        LOG.info(f"ReRanked score: {score} - {answers[ans]}")
-                    best_ans = reranked[0][1]
-
+                # semi-random pick, no re-ranker available
+                # ASSUMPTION: if skill took longer to process query, answer is more accurate
+                best_ans = list(answers.keys())[-1]
                 best = answers[best_ans]
+            else:
+                reranked = self.reranker.rerank(query.query,
+                                                list(answers.keys()),
+                                                lang=query.lang)
+                if self._min_reranker_score is None:
+                    # by default not set, the optimal value is reranker plugin specific
+                    candidates = reranked
+                else:
+                    candidates = [ans for ans in reranked if ans[0] >= self._min_reranker_score]
+                    for score, ans in [r for r in reranked if r not in candidates]:
+                        LOG.debug(f"ReRanker discarded low confidence answer: {score} - {answers[ans]}")
 
-            LOG.info('Handling with: ' + str(best['skill_id']))
-            query.selected_skill = best["skill_id"]
-            query.callback_data = {**best, "phrase": search_phrase}
-            query.answered = True
+                for score, ans in candidates:
+                    LOG.info(f"ReRanker score: {score} - {answers[ans]}")
+
+                if candidates:
+                    best_ans = candidates[0][1]
+                    best = answers[best_ans]
+                else:
+                    best = None
+
+            if best is not None:
+                LOG.info('Handling with: ' + str(best['skill_id']))
+                query.selected_skill = best["skill_id"]
+                query.callback_data = {**best, "phrase": search_phrase}
+                query.answered = True
+            else:
+                LOG.debug("No good answers from skills, not answering question")
+                query.answered = False
         else:
             query.answered = False
         query.completed.set()
